<?php
/*
Plugin Name: Import products and handle orders
Plugin URL: https://www.doubridge.com/
Description: Upload tens of thousands of product by ONE-CLICK, from preparing to shipment we will handle the rest for you.Don't worry about pics,descriptions,inventory,suppliers,finance etc, just focus on boosting sales and local service.
Version: 1.2.0
Author : DouBridge
Author URL: https://www.doubridge.com/
*/
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) :

if ( ! class_exists( 'Doubridge' ) ) :

class Doubridge {
	protected $logger;
	protected $http;

	public $DEBUG = false;
	public $SERVER_URL = 'https://app.doubridge.com';

	public function __construct() {		
		define( 'DOUBRIDGE_PATH', dirname( __FILE__ ) . '/' );
		define( 'DOUBRIDGE_BASENAME', plugin_basename( __FILE__ ) );
		$this->init();
		do_action( 'doubridge_loaded' );
	}

	protected function init() {
		require_once DOUBRIDGE_PATH . 'includes/log.php';
		require_once DOUBRIDGE_PATH . 'includes/http.php';
		require_once DOUBRIDGE_PATH . 'includes/functions.php';
		require_once DOUBRIDGE_PATH . 'includes/category.php';
		require_once DOUBRIDGE_PATH . 'includes/calculator.php';

		$this->logger   = new Doubridge_Log( 'debug.log' );
		$this->http		= new Doubridge_Http( $this->logger );
		$category 		= new Doubridge_Category( $this->http, $this->logger );
		$calculator     = new Doubridge_Calculator( $this->http, $this->logger );

		// freight
		add_action( 'woocommerce_before_calculate_totals', array( &$calculator, 'beforeCalculate' ) );
		// category
		add_action( 'create_product_cat', array( &$category, 'create' ) );
		add_action( 'edited_product_cat', array( &$category, 'edit' ) );
		add_action( 'pre_delete_term', array( &$category, 'delete' ) );
		add_action( 'init', array( &$this, 'callback' ) );

		add_filter( 'plugin_action_links_' . DOUBRIDGE_BASENAME, array( $this, 'plugin_action_links' ) );

		if ( is_admin() ) {
			add_action( 'admin_menu', array( &$this, 'admin_menu' ) );
			add_action( 'admin_init', array( &$this, 'register_settings' ) );
		}
	}

	public function register_settings() {
		register_setting( 'doubridge_api', 'doubridge_api' );
	}

	public function admin_menu() {
		add_options_page( 'Doubridge', 'Doubridge', 'manage_options', 'doubridge', array( &$this, 'settings_options' ) );
	}

	public function settings_options() {
		$options = get_option( 'doubridge_api' );

		if ( empty( $options['host'] ) ) {
			$options['host'] = $this->SERVER_URL;
		}

		if ( empty( $options['callback_url'] ) ) {
			$options['callback_url'] = home_url() . '/?api-callback';
		}

		// Securing Output
		$options = array_map( 'esc_html', $options );

		$token_url = home_url() . '/?api-token';

		include DOUBRIDGE_PATH . 'includes/settings.php';
	}

	public function callback() {
		if ( isset( $_GET['api-callback'] ) ) {
			
			if ( !empty( $_GET['code'] ) && !empty( $_GET['state'] ) ) {
				ob_end_clean();

				$data = array(
					'code' => esc_html( $_GET['code'] ),
					'state' => esc_html( $_GET['state'] )
				);

				header( 'Content-Type: application/json' );
				echo json_encode( $data );
			}

			exit;
		}

		if ( isset( $_GET['api-token'] ) ) {
			ob_end_clean();

			$host 			= sanitize_text_field( $_POST['doubridge_api']['host'] );
			$client_id 		= sanitize_text_field( $_POST['doubridge_api']['client_id'] );
			$client_secret 	= sanitize_text_field( $_POST['doubridge_api']['client_secret'] );
			$callback 		= esc_url_raw( $_POST['doubridge_api']['callback_url'] );

			header('content-type: application/json');

			$data = array();
			
			if ( !empty( $host ) && !empty( $client_id ) && !empty( $client_secret ) && !empty( $callback ) ) {				
				// Get code
				$host = trim( $host, '/' );
				$code = $this->http->getGrantCode( $host, $client_id, $callback );
				if ( $code ) {
					$result = $this->http->getToken( $host, $client_id, $client_secret, $callback, $code );

					if ( $result ) {
						$data = array(
							'error' => false,
							'errmsg' => '',
							'data' => $result
						);

						echo json_encode( $data );
						exit;
					}
				}
			}

			$data = array(
				'error' => true,
				'errmsg' => 'Failing',
				'data' => ''
			);
			
			echo json_encode( $data );
			exit;
		}
	}

	public function plugin_action_links( $links ) {
		$action_links = array(
			'settings' => '<a href="' . admin_url( 'options-general.php?page=doubridge' ) . '" aria-label="' . esc_attr__( 'View settings', 'woocommerce' ) . '">' . esc_html__( 'Settings', 'woocommerce' ) . '</a>',
		);

		return array_merge( $action_links, $links );
	}
}

$GLOBALS['doubridge'] = new Doubridge();

endif;
endif;
