<?php
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class Doubridge_Http {
    private $logger;
    
    public function __construct( $logger ) {
        $this->logger = $logger;
    }
    
    public function getGrantCode( $host, $client_id, $callback ) {
        $state = md5( 'doubirdge_api_state' . time() );

        $query_params = array(
            'state' => $state,
            'client_id' => $client_id,
            'response_type' => 'code',
            'approval_prompt' => 'auto',
            'redirect_uri' => $callback
        );

        $url = $host . '/oauth/authorize?' . http_build_query( $query_params );
        $result = $this->remoteGet( $url );

        if ( $result && isset( $result['state'] ) && $result['state'] == $state ) {
            return $result['code'];
        }

        return '';
    }

    public function getToken( $host, $client_id, $client_secret, $callback, $code ) {
        $data = array(
            'client_id' => $client_id,
            'client_secret' => $client_secret,
            'redirect_uri' => $callback,
            'grant_type' => 'authorization_code',
            'code' => $code
        );

        $url = $host . '/api/token';																	
        $result = $this->remotePost( $url, $data );

        if ( $result ) {
            $result['expires_time'] = time() + $result['expires_in'];

            //return base64_encode( json_encode( $result ) );
            return $result;
        }

        return '';
    }

    public function refreshToken( $host, $client_id, $client_secret, $callback, $refresh_token ) {
        $data = array(
            'client_id' => $client_id,
            'client_secret' => $client_secret,
            'redirect_uri' => $callback,
            'grant_type' => 'refresh_token',
            'refresh_token' => $refresh_token
        );

        $url = $host . '/api/token';																
        $result = $this->remotePost( $url, $data );

        if ( $result ) {
            $result['expires_time'] = time() + $result['expires_in'];

            //return base64_encode( json_encode( $result ) );
            return $result;
        }

        return '';
    }

    // 
    public function isTokenExpired( $tokens = array() ) {
        $expire = 1800;
        $now = time();
        if ( ( $tokens['expires_time'] + $expire ) <= $now ) {
            return true;
        }

        return false;
    }

    public function remoteGet( $url, $headers = array() ) {
        $args = array();

        if ( $headers ) {
            $args['headers'] = $headers;
        }

        $response = wp_remote_get( $url, $args );
        $http_code = wp_remote_retrieve_response_code( $response );

        if ( $http_code == 200 ) {
            $body = wp_remote_retrieve_body( $response );
            return json_decode( $body, true );
        }

        $this->logger->write( 'Get failing: ' . $url );
        $this->logger->write( $response );

        return array();
    }

    public function remotePost( $url, $data, $headers = array() ) {
        $args = array(
            'body' => $data
        );

        if ( $headers ) {
            $args['headers'] = $headers;
        }

        $response = wp_remote_post( $url, $args );
        $http_code = wp_remote_retrieve_response_code( $response );

        if ( $http_code == 200 ) {
            $body = wp_remote_retrieve_body( $response );
            return json_decode( $body, true );
        }

        $this->logger->write( 'Post failing: ' . $url );
        $this->logger->write( $response );

        return array();
    }

    public function remotePut( $url, $data, $headers = array() ) {
        $args = array(
            'method' => 'PUT',
            'body' => $data
        );

        if ( $headers ) {
            $args['headers'] = $headers;
        }

        $response = wp_remote_request( $url, $args );
        $http_code = wp_remote_retrieve_response_code( $response );
        
        if ( $http_code == 200 ) {
            $body = wp_remote_retrieve_body( $response );
            return json_decode( $body, true );
        } 

        $this->logger->write( 'Put failing: ' . $url );
        $this->logger->write( $response );

        return array();
    }

    public function remoteDelete( $url, $headers = array() ) {
        $args = array(
            'method' => 'DELETE'
        );
        
        if ( $headers ) {
            $args['headers'] = $headers;
        }

        $response = wp_remote_request( $url, $args );
        $http_code = wp_remote_retrieve_response_code( $response );
        
        if ( $http_code == 200 ) {
            return true;
        } 

        $this->logger->write( 'Delete failing: ' . $url );
        $this->logger->write( $response );

        return false;
    }
}
